==> 0.8.0.tar.gz: lib/acdart_router.dart <==

library ac_dart.addon_routebuilder;

import 'acdart_shelf_middleware.dart' show
  moustacheFile, productHostAuthenticator, sessionTokenAuthenticator,
  getSessionContext;
import 'package:shelf/shelf.dart' as shelf
  show Handler, Middleware, Response, Request, logRequests;
import 'package:shelf_route/shelf_route.dart' show HandlerAdapter;
import 'package:atlassian_connect_host/atlassian_connect_host.dart'
  show AcHostService, AcSessionContext, AcHostHttpClient, ConnectAddon;
import 'dart:async' show Future;
import 'package:logging/logging.dart' show Logger;
import 'package:atlassian_connect_host/host_event_model.dart'
  show AddonHost, InstallationEvent, AcHostRegistration;
import 'package:path/path.dart' as path show join;
import 'package:atlassian_connect_shelf/src/util.dart'
  show pathRelativeToAppRoot;
import 'package:shelf_bind/shelf_bind.dart' as b
  show CustomObjectFactory, RequestBody, handlerAdapter;
import 'package:constrain/constrain.dart';
import 'dart:convert';
import 'package:mojito/src/router_impl.dart';

final Logger _log = new Logger('AcDartAddonRouteBuilder');

//typedef ConnectAddon DescriptorFactory(AddonHost addonHost);
const String _DESCRIPTOR_PATH = '/atlassian-connect.json';

final Validator _validator =  new Validator(); // TODO: this is not a shared instance!!!

final JsonEncoder _prettyJson = new JsonEncoder.withIndent('  ');

class AcDartRouter extends RouterImpl {
  final AddonHost _addonHost;
  final String _assetPath;


  AcDartRouter._internal(HandlerAdapter handlerAdapter,
      this._addonHost, this._assetPath)
      : super(handlerAdapter: handlerAdapter);

  factory AcDartRouter(AcHostService hostService, AddonHost addonHost,
               { bool logRequests: true, String assetPath,
                 Map<Type, b.CustomObjectFactory> customObjects }) {

    final _customObjects = <Type, b.CustomObjectFactory>{};
    if (customObjects != null) {
      _customObjects.addAll(customObjects);
    }

    _customObjects.addAll({
        AcDartRequestContext: (request) => _context(hostService, request), 
        AcHostService: (_) => hostService,
        AcSessionContext: getSessionContext,
        AcHostHttpClient: (req) => _context(hostService, req).httpClient,
        AcHostRegistration: (req) => _context(hostService, req).hostRegistration
    });

    final _handlerAdapter = b.handlerAdapter(customObjects: _customObjects,
        // false till https://bitbucket.org/andersmholmgren/constraint/issue/34/constraint-resolver-does-not-cope-with-all
        validateParameters: false);

    final rootRouter = new AcDartRouter._internal(
        _handlerAdapter, addonHost,
        (assetPath != null ? assetPath : _assetDir));

    // TODO: should support middleware directly in router ctr
//    return logRequests ? rootRouter.child('/',
//        middleware: shelf.logRequests(logger: _requestLogger))
//        : rootRouter;

    return rootRouter;
  }

  static AcDartRequestContext _context(AcHostService hostService, request) => 
      new AcDartRequestContext(hostService, request);

  /// adds a route to handle the installation lifecycle event. You can override
  /// the [installationPath] if you desire but it is discouraged
  void addInstallationRoute([String installationPath = '/installed']) {
    post(installationPath, _installationHandler);
  }

  /// adds a route to serve the `atlassian-connect.json` file from the standard
  /// location. You can provide additional variables that you can use in the
  /// moustache template.
  void addDescriptorTemplateRoute({
    Map<String, String> additionalVariables: const {} }) {
    get(_DESCRIPTOR_PATH, _descriptorTemplateHander(additionalVariables));
  }

  void addDescriptorRoute(ConnectAddon addon) {
    final violations = _validator.validate(addon);
    if (violations.isNotEmpty) {
      final message = violations.map((v) => v.verboseMessage).join('\n');
      throw new ArgumentError('Invalid Addon\n$message');
    }
    get(_DESCRIPTOR_PATH, () => addon);
  }
  
  Future<shelf.Response> _installationHandler(
      @b.RequestBody() InstallationEvent installationEvent,
      AcHostService hostService) {
    _log.fine('received installation request '
        '${_prettyJson.convert(installationEvent)}');

    final registrationFuture = hostService.registerHost(installationEvent);

    return registrationFuture.then((r) {
      return new shelf.Response.ok("installed");
    }, onError: (e, stackTrace) {
      _log.warning('Registration failed', e, stackTrace);
      // TODO: throw instead and let shelf_exeption_route
      // TODO: more specific code and message. e.g 404, 401, 403
      return new shelf.Response(400, body: 'Registration failed');
    });
  }

  shelf.Handler _descriptorTemplateHander(
        Map<String, String> additionalVariables) {
    final templateVariables = {
      "addonKey": _addonHost.key,
      "baseUrl": _addonHost.baseUrl.toString()
    }..addAll(additionalVariables);

    return moustacheFile(path.join(_assetPath, 'atlassian-connect.json'),
        templateVariables: templateVariables);
  }

}

class AcDartRequestContext {
  final AcHostService hostService;
  final shelf.Request request;
  
  AcDartRequestContext(this.hostService, this.request);
  
  AcSessionContext get sessionContext => getSessionContext(request);
  
  Future<AcHostHttpClient> get httpClient =>
      hostService.httpClientForHostKey(sessionContext.hostKey);
  
  Future<AcHostRegistration> get hostRegistration => 
      hostService.findRegistrationByHostKey(sessionContext.hostKey)
      .then((regOpt) => regOpt.getOrElse(() => null));
}
void _requestLogger(String msg, bool isError) {
  if (isError) {
    _log.warning(msg);
  }
  else {
    _log.finest(msg);
  }
}

final String _assetDir = pathRelativeToAppRoot('asset');


Iterable<Map> _iterableToJson(Iterable i) =>
    i == null ? null : i.map((j) => j.toJson()).toList(growable: false);

==> 0.10.0.tar.gz: lib/acdart_router.dart <==

library ac_dart.addon_routebuilder;

import 'acdart_shelf_middleware.dart' show
  moustacheFile, productHostAuthenticator, sessionTokenAuthenticator,
  getSessionContext;
import 'package:shelf/shelf.dart'
  show Handler, Middleware, Response, Request, logRequests;
import 'package:shelf_route/shelf_route.dart';
import 'package:atlassian_connect_host/atlassian_connect_host.dart'
  show AcHostHttpClient, AcHostService, AcSessionContext, ConnectAddon, HcInstallationEvent;
import 'dart:async' show Future;
import 'package:logging/logging.dart' show Logger;
import 'package:atlassian_connect_host/host_event_model.dart';
import 'package:path/path.dart' as path show join;
import 'package:atlassian_connect_shelf/src/util.dart'
  show pathRelativeToAppRoot;
import 'package:shelf_bind/shelf_bind.dart' as b
  show CustomObjectFactory, RequestBody, handlerAdapter;
import 'package:constrain/constrain.dart';
import 'dart:convert';
import 'package:mojito/src/router_impl.dart';
import 'package:shelf_exception_response/exception.dart';

final Logger _log = new Logger('AcDartAddonRouteBuilder');

//typedef ConnectAddon DescriptorFactory(AddonHost addonHost);
const String _DESCRIPTOR_PATH = '/atlassian-connect.json';

final Validator _validator =  new Validator(); // TODO: this is not a shared instance!!!

final JsonEncoder _prettyJson = new JsonEncoder.withIndent('  ');

class AcDartRouter extends RouterImpl {
  final AddonHost _addonHost;
  final String _assetPath;


  AcDartRouter._internal(HandlerAdapter handlerAdapter,
      this._addonHost, this._assetPath, {path, middleware})
      : super(handlerAdapter: handlerAdapter, path: path,
          middleware: middleware);

  factory AcDartRouter(AcHostService hostService, AddonHost addonHost,
               { bool logRequests: true, String assetPath,
                 Map<Type, b.CustomObjectFactory> customObjects }) {


    final _customObjects = <Type, b.CustomObjectFactory>{};
    if (customObjects != null) {
      _customObjects.addAll(customObjects);
    }

    _customObjects.addAll({
        AcDartRequestContext: (request) => _context(hostService, request), 
        AcHostService: (_) => hostService,
        AcSessionContext: getSessionContext,
        AcHostHttpClient: (req) => _context(hostService, req).httpClient,
        AcHostRegistration: (req) => _context(hostService, req).hostRegistration
    });

    final _handlerAdapter = b.handlerAdapter(customObjects: _customObjects,
        // false till https://bitbucket.org/andersmholmgren/constraint/issue/34/constraint-resolver-does-not-cope-with-all
        validateParameters: false);

    final rootRouter = new AcDartRouter._internal(
        _handlerAdapter, addonHost,
        (assetPath != null ? assetPath : _assetDir));

    return rootRouter;
  }

  static AcDartRequestContext _context(AcHostService hostService, request) => 
      new AcDartRequestContext(hostService, request);

  /// adds a route to handle the installation lifecycle event. You can override
  /// the [installationPath] if you desire but it is discouraged
  void addInstallationRoute([String installationPath = '/installed']) {
    post(installationPath, _installationHandler);
  }
    
  // TODO: consolidate
  void addHcInstallationRoute([String installationPath = '/installed']) {
    post(installationPath, _hcInstallationHandler);
  }

  /// adds a route to serve the `atlassian-connect.json` file from the standard
  /// location. You can provide additional variables that you can use in the
  /// moustache template.
  void addDescriptorTemplateRoute({
    Map<String, String> additionalVariables: const {} }) {
    get(_DESCRIPTOR_PATH, _descriptorTemplateHander(additionalVariables));
  }

  void addDescriptorRoute(ConnectAddon addon) {
    final violations = _validator.validate(addon);
    if (violations.isNotEmpty) {
      final message = violations.map((v) => v.verboseMessage).join('\n');
      throw new ArgumentError('Invalid Addon\n$message');
    }
    get(_DESCRIPTOR_PATH, () => addon);
  }
  
  Future<Response> _installationHandler(
      @b.RequestBody() InstallationEvent installationEvent,
      AcHostService hostService) async {
    _log.fine('received installation request '
        '${_prettyJson.convert(installationEvent)}');

    try {
      await hostService.registerHost(installationEvent);
   
      return new Response.ok("installed");
    } catch(e, stackTrace) {
      _log.warning('Registration failed', e, stackTrace);
      // TODO: more specific code and message. e.g 404, 401, 403
      throw new BadRequestException({}, 'Regstration Failed');
    };
  }

  Future<Response> _hcInstallationHandler(
      @b.RequestBody() HcInstallationEvent installationEvent,
      AcHostService hostService) async
      => await _installationHandler(installationEvent, hostService);

  Handler _descriptorTemplateHander(
        Map<String, String> additionalVariables) {
    final templateVariables = {
      "addonKey": _addonHost.key,
      "baseUrl": _addonHost.baseUrl.toString()
    }..addAll(additionalVariables);

    return moustacheFile(path.join(_assetPath, 'atlassian-connect.json'),
        templateVariables: templateVariables);
  }

  @override
  RouterImpl createChild(HandlerAdapter handlerAdapter,
                         RouteableAdapter routeableAdapter,
                         PathAdapter pathAdapter, path,
                         Middleware middleware) {

    // TODO this is broken. Need to make use of all the passed in params
    return new AcDartRouter._internal(handlerAdapter, _addonHost, 
        _assetPath, path: path, middleware: middleware);
  }

}

class AcDartRequestContext {
  final AcHostService hostService;
  final Request request;
  
  AcDartRequestContext(this.hostService, this.request);
  
  AcSessionContext get sessionContext => getSessionContext(request);
  
  Future<AcHostHttpClient> get httpClient =>
      hostService.httpClientForHostKey(sessionContext.hostKey);
  
  Future<AcHostRegistration> get hostRegistration => 
      hostService.findRegistrationByHostKey(sessionContext.hostKey)
      .then((regOpt) => regOpt.getOrElse(() => null));
}


final String _assetDir = pathRelativeToAppRoot('asset');
